%% Comparison of BWS systems using dynamic similarity
%  Gait dynamics are represented using Froude, Strouhal, and Modela-W
%  numbers. These numbers require the leg length, gait speed, and cadence 
%  for computation.
%  Gait data up to 35% BWS is considered
%  Reduction in dynamic similarity (delDS) is quantified using RMS
%  difference w.r.t. the 0% BWS condition
%  SW:   Simplest walking model
%  SLIP: Spring-loaded inverted pendulum model
%  MR:   Muslce reflex model
%  Function 'calcDynSim.m' is used to calculate the above-mentioned rmse values
%  - Salil Apte, Nov 2019

clear all
clc
bws = 0:5:35;
r1 = [228 26 28]./255; % red    - human experimental data for treadmill healthy group
b1 = [55 126 184]./255;% blue   - human experimental data for overground healthy group
g1 = [77 175 74]./255; % green  - muscle reflex model data
o1 = [255 127 0]./255; % orange - SLIP model data
y1 = [255 255 51]./255;% yellow - simplest walking model data
l1 = [152 78 163]./255;% unused
%% Simplest walking model (SW)
% Gait data: leg length (in m), cadence (C), and walking speed (WS)
leg_SW = 1;

% Constant force (CF) BWS 
C_SW_CF       = [2.774;2.754;2.815;0;0;0;0;0];
WS_SW_CF       = [2.48;2.47;2.57;0;0;0;0;0];
% Dimensionless gait dynamics
[NFr_SW_CF,Str_SW_CF,Mod_SW_CF,delDS_SW_CF] = calcDynSim(WS_SW_CF,C_SW_CF,leg_SW,10);

% Counterweight BWS
C_SW_CW       = [2.774;2.811;0;0;0;0;0;0];
WS_SW_CW       = [2.48;2.58;0;0;0;0;0;0];
% Dimensionless gait dynamics
[NFr_SW_CW,Str_SW_CW,Mod_SW_CW,delDS_SW_CW] = calcDynSim(WS_SW_CW,C_SW_CW,leg_SW,5);

% Tuned spring BWS
C_SW_TS       = [2.774;2.754;2.832;0;0;0;0;0];
WS_SW_TS       = [2.48;2.48;2.52;0;0;0;0;0];
% Dimensionless gait dynamics
[NFr_SW_TS,Str_SW_TS,Mod_SW_TS,delDS_SW_TS] = calcDynSim(WS_SW_TS,C_SW_TS,leg_SW,10);

%% SLIP model
% Gait data: leg length (in m), cadence (C), and walking speed (WS)
leg_SLIP = 1;

% Constant force (CF) BWS 
C_SLIP_CF     = [1.6667;1.6000;1.6000;1.6000;1.5333;1.5333;1.4467;1.4667];
WS_SLIP_CF     = [1.1772;1.1604;1.1720;1.1637;1.1420;1.1354;1.1139;1.1091];
% Dimensionless gait dynamics
[NFr_SLIP_CF,Str_SLIP_CF,Mod_SLIP_CF,delDS_SLIP_CF] = calcDynSim(WS_SLIP_CF,C_SLIP_CF,leg_SLIP,35);

% Counterweight BWS
C_SLIP_CW     = [1.6667;1.6000;1.5333;1.4667;1.4000;1.3333;1.3333;1.2000];
WS_SLIP_CW     = [1.1772;1.1403;1.1199;1.0767;1.0310;0.9922;0.9853;0.9109]; 
% Dimensionless gait dynamics
[NFr_SLIP_CW,Str_SLIP_CW,Mod_SLIP_CW,delDS_SLIP_CW] = calcDynSim(WS_SLIP_CW,C_SLIP_CW,leg_SLIP,35);

% Tuned spring BWS
C_SLIP_TS     = [1.6667;1.6000;1.6000;1.6000;1.5333;1.5333;1.5333;1.4667];
WS_SLIP_TS     = [1.1772;1.1647;1.1827;1.1835;1.1690;1.1749;1.1602;1.1735];
% Dimensionless gait dynamics
[NFr_SLIP_TS,Str_SLIP_TS,Mod_SLIP_TS,delDS_SLIP_TS] = calcDynSim(WS_SLIP_TS,C_SLIP_TS,leg_SLIP,35);

%% Muscle-reflex model (MR)
% Gait data: leg length (in m), cadence (C), and walking speed (WS)
leg_MR = 1; % shank length (0.5) + thigh length (0.5)

% Constant force (CF) BWS 
C_MR_CF       = [1.7149;1.6327;1.5038;1.4414;1.4035;1.3746;1.3536;1.4109];
WS_MR_CF       = [1.3626;1.2864;1.1550;1.0825;1.0368;1.0080;0.9134;0.9020];
% Dimensionless gait dynamics
[NFr_MR_CF,Str_MR_CF,Mod_MR_CF,delDS_MR_CF] = calcDynSim(WS_MR_CF,C_MR_CF,leg_MR,35);

% Counterweight (CW) BWS
C_MR_CW       = [1.7241;1.5936;1.4842;1.4311;1.3889;1.3532;1.3280;1.3063];
WS_MR_CW       = [1.3720;1.2448;1.1242;1.0677;1.0242;0.9844;0.9343;0.9057];
% Dimensionless gait dynamics
[NFr_MR_CW,Str_MR_CW,Mod_MR_CW,delDS_MR_CW] = calcDynSim(WS_MR_CW,C_MR_CW,leg_MR,35);

% Tuned spring (TS) BWS
C_MR_TS       = [1.7112;1.6931;1.6771;1.4519;1.4172;1.3944;1.3617;1.3569]; % low-pass filtered
WS_MR_TS       = [1.3614;1.3551;1.3348;1.1140;1.0629;1.0180;0.9679;0.9628]; 
% Dimensionless gait dynamics
[NFr_MR_TS,Str_MR_TS,Mod_MR_TS,delDS_MR_TS] = calcDynSim(WS_MR_TS,C_MR_TS,leg_MR,35);

%% Dynamic similarity plots (Modela-W)

mod_SW = [Mod_SW_CF./10,Mod_SW_CW./10,Mod_SW_TS./10];
mod_SLIP = [Mod_SLIP_CF,Mod_SLIP_CW,Mod_SLIP_TS];
mod_MR = [Mod_MR_CF,Mod_MR_CW,Mod_MR_TS];

figure 
bar(mod_SW)
grid on
grid minor
% axis([0 40 0 0.05])
set(gca,'fontsize',36,'fontname','times')
% xticks([0 10 20 30 40 50 60 70 80])
% xticklabels({'0', '5', '20', '30', '40'})
ylabel('Modela-W','FontSize',36,'FontWeight','bold')
xlabel('Percentage Body weight unloading (BWU)','FontSize',36,'FontWeight','bold')

figure 
bar(mod_SLIP)
hold on
grid on
grid minor
% axis([0 40 0.02 0.08])
set(gca,'fontsize',36,'fontname','times')
% xticks([0 10 20 30 40 50 60 70 80])
% xticklabels({'0', '5', '20', '30', '40'})
ylabel('Modela-W','FontSize',36,'FontWeight','bold')
xlabel('Percentage Body weight unloading (BWU)','FontSize',36,'FontWeight','bold')

figure 
hold on
bar(mod_MR)
grid on
grid minor
% axis([0 40 0.03 0.09])
set(gca,'fontsize',36,'fontname','times')
% xticks([0 10 20 30 40 50 60 70 80])
% xticklabels({'0', '5', '20', '30', '40'})
ylabel('Modela-W','FontSize',36,'FontWeight','bold')
xlabel('Percentage Body weight unloading (BWU)','FontSize',36,'FontWeight','bold')

%% Range (maximum - minimum) of feasible BWS values (excluding 0%) for different initial gait speeds
iws = 1:1:15;
range_SW_CF = [0;0;2;2;2;2;2;2;1;1;0;0;0;0;0]*5;
range_SW_CW = [0;0;1;1;1;1;1;1;1;1;0;0;0;0;0]*5;
range_SW_TS = [0;0;2;2;2;2;2;2;1;2;0;0;0;0;0]*5;

range_SLIP_CF = [0;0;0;0;7;6;5;3;1;0;0;0;0;0;0]*5;
range_SLIP_CW = [0;0;4;3;1;1;1;0;0;0;0;0;0;0;0]*5;
range_SLIP_TS = [0;0;0;0;3;5;7;7;6;0;0;0;0;0;0]*5;

range_MR_CF = [3;3;2;3;3;7;8;8;7;8;7;5;5;2;2]*5;
range_MR_CW = [1;4;2;3;3;2;3;4;5;4;3;3;6;4;5]*5;
range_MR_TS = [9;9;10;10;10;10;10;10;10;10;10;10;8;8;3]*5;

% Plots
figure 
hold on
plot(iws,range_SW_CF,'--d',iws, range_SW_CW,':o',iws,range_SW_TS,'-x','linewidth',2,'Markersize',20,'Color',l1)
grid on
grid minor
axis([-1 16 0 55])
set(gca,'fontsize',36,'fontname','times')
xticks([1 2 3 4 5 6 7 8 9 10 11 12 13 14 15])
xticklabels({'0.2167'	''	''	''	'0.6167'	''	''	'0.7167'	''	''	'0.8167'	''	''	''	'1.2167'})
ylabel('Range of feasible BWS','FontSize',36,'FontWeight','bold')
xlabel('Initial walking speed','FontSize',36,'FontWeight','bold')

figure 
hold on
plot(iws,range_SLIP_CF,'--d',iws, range_SLIP_CW,':o',iws,range_SLIP_TS,'-x','linewidth',2,'Markersize',20,'Color',o1)
grid on
grid minor
axis([-1 16 0 55])
set(gca,'fontsize',36,'fontname','times')
xticks([1 2 3 4 5 6 7 8 9 10 11 12 13 14 15])
xticklabels({'0.6185'	''	''	''	'1.0185'	''	''	'1.1185'	''	''	'1.2185'	''	''	''	'1.6185'})
ylabel('Range of feasible BWS','FontSize',36,'FontWeight','bold')
xlabel('Initial walking speed','FontSize',36,'FontWeight','bold')

figure 
hold on
plot(iws,range_MR_CF,'--d',iws, range_MR_CW,':o',iws,range_MR_TS,'-x','linewidth',2,'Markersize',20,'Color',g1)
grid on
grid minor
axis([-1 16 0 55])
set(gca,'fontsize',36,'fontname','times')
xticks([1 2 3 4 5 6 7 8 9 10 11 12 13 14 15])
xticklabels({'0.8'	''	''	''	'1.2'	''	''	'1.3'	''	''	'1.4'	''	''	''	'1.8'})
ylabel('Range of feasible BWS','FontSize',36,'FontWeight','bold')
xlabel('Initial walking speed','FontSize',36,'FontWeight','bold')